
//   Copyright 2022 Esri
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

#include "Display_a_map.h"
#include "ArcGISRuntimeEnvironment.h"
#include "Basemap.h"
#include "Map.h"

#include "MapQuickView.h"
#include <QUrl>

#include "FeatureLayer.h"
#include "SpatialReference.h"
#include "ServiceFeatureTable.h"
#include "Viewpoint.h"
#include "Point.h"
#include "QueryParameters.h"
#include "FeatureQueryResult.h"
#include "Feature.h"
#include "SelectionProperties.h"
#include <QColor>
#include <QList>
#include <memory>

using namespace Esri::ArcGISRuntime;

Display_a_map::Display_a_map(QObject* parent /* = nullptr */):
    QObject(parent),
    m_map(new Map(BasemapStyle::ArcGISTopographic, this))
{
}

Display_a_map::~Display_a_map()
{
}

MapQuickView* Display_a_map::mapView() const
{
    return m_mapView;
}

void Display_a_map::setupViewpoint()
{
    const Point center(-118.80543, 34.02700, SpatialReference::wgs84());
    const Viewpoint viewpoint(center, 100000.0);
    m_mapView->setViewpoint(viewpoint);
}

void Display_a_map::addFeatureLayer()
{
    // Create a ServiceFeatureTable from a QUrl.
    m_featureTable = new ServiceFeatureTable(QUrl("https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/LA_County_Parcels/FeatureServer/0"), this);
    // Create the feature layer using the feature table.
    m_featureLayer = new FeatureLayer(m_featureTable, this);
    // Add the feature layer to the map.
    m_map->operationalLayers()->append(m_featureLayer);
}

void Display_a_map::runQuery(const QString& expression)
{
    if (!m_featureTable)
        return;

    // Create a query parameter object and set the WhereClause.
    QueryParameters queryParams;
    queryParams.setGeometry(m_mapView->currentViewpoint(ViewpointType::BoundingGeometry).targetGeometry().extent());
    queryParams.setWhereClause(expression);
    m_featureTable->queryFeatures(queryParams);
}

void Display_a_map::selectFeatures()
{
    connect(m_featureTable, &ServiceFeatureTable::queryFeaturesCompleted, this, [this](QUuid, FeatureQueryResult* rawQueryResult)
    {
        if (!rawQueryResult)
            return;

        // Wrap the FeatureQueryResult in a unique_ptr to manage the memory effectively.
        auto queryResult = std::unique_ptr<FeatureQueryResult>(rawQueryResult);

        // Clear any existing selection.
        m_featureLayer->clearSelection();
        QList<Feature*> features;

        // Iterate over the result object.
        while(queryResult->iterator().hasNext())
        {
            Feature* feature = queryResult->iterator().next(this);
            // Add each feature to the list.
            features.append(feature);
        }
        // Select features.
        m_featureLayer->selectFeatures(features);
    });
}

// Set the view (created in QML)
void Display_a_map::setMapView(MapQuickView* mapView)
{
    if (!mapView || mapView == m_mapView)
    {
        return;
    }
    m_mapView = mapView;
    m_mapView->setMap(m_map);

    addFeatureLayer();
    selectFeatures();
    m_mapView->setSelectionProperties(SelectionProperties(QColor(Qt::yellow)));

    setupViewpoint();
    emit mapViewChanged();
}

